#include "sn76489.h"

sn76489::sn76489( byte we, byte ce, byte rdy ) {
  _we = we;
  _ce = ce;
  _rdy = rdy;
}

void sn76489::init() {
  DDRA = 0xFF;
  PORTA = 0x00;
  pinMode( _rdy, INPUT_PULLUP );
  pinMode( _ce, OUTPUT );
  pinMode( _we, OUTPUT );
  digitalWrite( _ce , LOW );
  digitalWrite( _we , LOW );
  _send( 0x9F ); //4 byte silence sn sequence
  _send( 0xBF );
  _send( 0xDF );
  _send( 0xFF );
}

void sn76489::ping() {
  setFreq(0, 392); // Sol
  delay(100);
  setReg(0x01, ATTENUATION_OFF);
  delay(100);
  setFreq(1, 440); // La
  delay(100);
  setReg(0x03, ATTENUATION_OFF);
  delay(100);
  setFreq(2, 493); // Si
  delay(100);
  setReg(0x05, ATTENUATION_OFF);
  delay(100);
  setReg(0x06, NOISE_WHITE_FEEDBACK | NOISE_SHIFT_1024);
  delay(100);
  setReg(0x07, ATTENUATION_OFF);
  delay(100);
  setReg(0x01, ATTENUATION_2DB);
  delay(100);
  setReg(0x01, ATTENUATION_OFF);
  delay(100);
  setReg(0x03, ATTENUATION_2DB);
  delay(100);
  setReg(0x03, ATTENUATION_OFF);
  delay(100);
  setReg(0x05, ATTENUATION_2DB);
  delay(100);
  setReg(0x05, ATTENUATION_OFF);
  delay(100);
  setReg(0x07, ATTENUATION_2DB);
  delay(100);
  setReg(0x07, ATTENUATION_OFF);
}

void sn76489::_send ( byte data ) {
  PORTA = data;
  digitalWrite( _ce , LOW );
  digitalWrite( _we , LOW );
  while ( !digitalRead( _rdy ));
  digitalWrite( _ce , HIGH );
  digitalWrite( _we , HIGH );
}

void sn76489::setReg ( uint8_t reg, uint8_t data ) {
  _send( 0x80 | (( reg & 0x07 ) << 4 ) | ( data & 0x0F ));
}

void sn76489::setFreq ( uint8_t channel, uint16_t frequency ) { //freq in Hz
  uint16_t f = FREG_REG_VAL( frequency );
  setReg( 2 * channel, f & 0x0F );
  _send(( f >> 4 ) & 0x3F );
}

void sn76489::test() {
  setFreq(0, 392); // Sol
  delay(100);
  setReg(0x01, ATTENUATION_OFF);
  delay(100);
  setFreq(1, 440); // La
  delay(100);
  setReg(0x03, ATTENUATION_OFF);
  delay(100);
  setFreq(2, 493); // Si
  delay(100);
  setReg(0x05, ATTENUATION_OFF);
  delay(100);
  setReg(0x06, NOISE_WHITE_FEEDBACK | NOISE_SHIFT_1024);
  delay(100);
  setReg(0x07, ATTENUATION_OFF);
  while ( 1 ) {
    delay(1000);
    setReg(0x01, ATTENUATION_2DB);
    delay(1000);
    setReg(0x01, ATTENUATION_OFF);
    delay(1000);
    setReg(0x03, ATTENUATION_2DB);
    delay(1000);
    setReg(0x03, ATTENUATION_OFF);
    delay(1000);
    setReg(0x05, ATTENUATION_2DB);
    delay(1000);
    setReg(0x05, ATTENUATION_OFF);
    delay(1000);
    setReg(0x07, ATTENUATION_2DB);
    delay(1000);
    setReg(0x07, ATTENUATION_OFF);
  }
}
